import { IsEmail, IsNotEmpty, IsObject, IsOptional, IsPhoneNumber, IsString } from 'class-validator';
import { getConnectionManager } from 'typeorm';
import { ParticipantAddressNotFoundError, ParticipantAddressWrongTypeError } from '../../errors/ParticipantErrors';
import { Address } from '../entities/Address';
import { CreateAddress } from './CreateAddress';

export abstract class CreateParticipant {
    /**
     * The new participant's first name.
     */
    @IsString()
    @IsNotEmpty()
    firstname: string;

    /**
     * The new participant's middle name.
     * Optional.
     */
    @IsString()
    @IsNotEmpty()
    middlename?: string;

    /**
     * The new participant's last name.
     */
    @IsString()
    @IsNotEmpty()
    lastname: string;

    /**
     * The new participant's phone number.
     * Optional.
     */
    @IsString()
    @IsOptional()
    @IsPhoneNumber("ZZ")
    phone?: string;

    /**
     * The new participant's e-mail address.
     * Optional.
     */
    @IsString()
    @IsOptional()
    @IsEmail()
    email?: string;

    /**
     * The new participant's address.
     * Must be of type number (address id), createAddress (new address) or address (existing address)
     * Optional.
     */
    @IsObject()
    address?: number | CreateAddress | Address;

    /**
     * Creates a Participant entity from this.
     */
    public async getAddress(): Promise<Address> {
        if (this.address === undefined) {
            return null;
        }
        if (this.address! instanceof Address) {
            return this.address;
        }
        if (this.address! instanceof CreateAddress) {
            return this.address.toAddress();
        }
        if (!isNaN(this.address)) {
            let address = await getConnectionManager().get().getRepository(Address).findOne({ id: this.address });
            if (!address) { throw new ParticipantAddressNotFoundError; }
            return address;
        }

        throw new ParticipantAddressWrongTypeError;
    }
}